//
// TBMK.cxx, v1.0, Thu Dec  6 14:17:04 CET 2007
// Author: P.Konczykowski
//

#include <iostream>
#include "TBMK.h"
#include "TGuzey.h"
#include "TLorentzVector.h"
using namespace std;

ClassImp(TBMK)

////////////////////////////////////////////////////////////////////////////////
//
// BMK Class
// 
////////////////////////////////////////////////////////////////////////////////

//_____________________________________________________________________________

 TBMK::TBMK()
{
  // Default Constructor

  cout << " TBMK default constructor called" << endl << endl;
  Init(5.75);

}

//_____________________________________________________________________________

 TBMK::TBMK(Double_t Ebeam)
{

  cout << " TBMK constructor called" << endl << endl;
  Init(Ebeam);

};

//_____________________________________________________________________________

 TBMK::~TBMK()
{

  cout << " TBMK destructor called" << endl << endl;

}

//_____________________________________________________________________________

void TBMK::Init(Double_t Ebeam)
{
  Eb=Ebeam;
  M=0.938271998; // Proton !
  alpha=7.297352533e-3;
  kin_init=kFALSE;
  BHUnpC_init=kFALSE;
  IUnpC_init=kFALSE;
  DVCSUnpC_init=kFALSE;
  BH_LC_init=kFALSE;
  I_LC_init=kFALSE;
  DVCS_LC_init=kFALSE;
  BH_TC_init=kFALSE;
  I_TC_init=kFALSE;
  DVCS_TC_init=kFALSE;

  cout << " Ebeam=" << Eb << endl<< endl;

}

//_____________________________________________________________________________

void TBMK::Kin(Double_t xh, Double_t Q2h, Double_t th)
{
 //   cout << " New kinematics" << endl;

  Double_t nu;
  Double_t s;
  Double_t Ep;
  Double_t thl;
  Double_t ql;
  Double_t q0;
  Double_t qp0;
  Double_t p0;
  Double_t pp0;
  Double_t p1cm;
  Double_t p3cm;
  Double_t thppcm;

  TLorentzVector q1;

  TGuzey dum;
  Double_t GPD[4];

  Ht_Im=0;
  Ht_Re=0;
  Et_Im=0;
  Et_Re=0;


  xb=xh;
  Q2=Q2h;
  t=th;

  varphi=0;

  eps=2.*xb*M/sqrt(Q2);
  nu=Q2/(2.*M*xb);
  s=-Q2+M*M+2.*M*nu;
  Ep=Eb-Q2/(2.*M*xb);

  thl=2*asin(sqrt(Q2/(4*Eb*Ep)));
  ql=sqrt(Eb*Eb+Ep*Ep-2*Eb*Ep*cos(thl));
  thph=asin(-Ep*sin(thl)/ql);
  q0=(s-Q2-M*M)/(2*sqrt(s));
  qp0=(s-M*M)/(2*sqrt(s));

  p0=(s+Q2+M*M)/(2*sqrt(s));
  pp0=(s+M*M)/(2*sqrt(s));
  
  p1cm=sqrt(Q2+q0*q0);
  p3cm=qp0;

  t0=Q2*Q2/(4*s)-(p1cm-p3cm)*(p1cm-p3cm);
  t1=Q2*Q2/(4*s)-(p1cm+p3cm)*(p1cm+p3cm);

//  if(!(t1<t&&t<t0)) {
//    cout << " t is not in the physical region !" << endl;
//    cout<<" t: "<<t<<"  t0: "<<t0<<"  t1: "<<t1<<endl;
//    exit(-1);
//  }

  xbmin=2*Eb*Q2/(M*(4*Eb*Eb-Q2));
  xbmax=Q2/(Q2+4-M*M);

//  if(!(xbmin<xb&&xb<xbmax)) {
//    cout << " xb is not in the physical region !" << endl;
//    cout<<" xb: "<<xb<<"  xbmin: "<<xbmin<<"  xbmax: "<<xbmax<<endl;
//    exit(-1);
//  }

  thppcm=2*asin(sqrt((Q2*Q2/(4*s)-(p1cm-p3cm)*(p1cm-p3cm)-t)/(4*p1cm*p3cm)));
  
  b.SetXYZ(0,0,ql/(nu+M));
  
  //Ref du labo
  k.SetPxPyPzE(0,0,Eb,Eb);
  kp.SetPxPyPzE(Ep*sin(thl),0,Ep*cos(thl),Ep); 
  p1.SetPxPyPzE(0,0,0,M);
  q1.SetPxPyPzE(ql*sin(thph),0,ql*cos(thph),nu);
  
  //Ref du cdm
  q2.SetPxPyPzE(qp0*sin(thppcm),0,qp0*cos(thppcm),qp0);
  p2.SetPxPyPzE(-qp0*sin(thppcm),0,-qp0*cos(thppcm),pp0);
  
  //Ref du photon virtuel
  q2.Boost(b);
  p2.Boost(b);
  
  //on trouve enfin les composantes des 4-vect des pces sortantes dans le ref du lab
  q2.RotateY(thph);
  p2.RotateY(thph);

  y=(p1*q1)/(p1*k);

  K2=-t/Q2*(1-xb)*(1-y-y*y*eps*eps/4)*(1-t0/t)*(sqrt(1+eps*eps)+(xb*(1-xb)+eps*eps/4)*(t-t0)/(Q2*(1-xb)));

 // cout<<" y: "<<y<<"  K2: "<<K2<<endl<<endl;

  // Get GPDs from Guzey

  dum.GetGPD(xb,Q2,t,GPD);
  H_Im=GPD[0];
  H_Re=GPD[1];
  E_Im=GPD[2];
  E_Re=GPD[3];

  // Set flags

  kin_init=kTRUE;
  BHUnpC_init=kFALSE;
  DVCSUnpC_init=kFALSE;
  IUnpC_init=kFALSE;
  BH_LC_init=kFALSE;
  I_LC_init=kFALSE;
  DVCS_LC_init=kFALSE;
  BH_TC_init=kFALSE;
  I_TC_init=kFALSE;
  DVCS_TC_init=kFALSE;
}

//_____________________________________________________________________________

void TBMK::BHUnpC()
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!BHUnpC_init) {

    Double_t F1=F1p(t);  // for now proton target
    Double_t F2=F2p(t);
    
    cunp_bh[0]=8*K2*((2+3*eps*eps)*Q2/t*(F1*F1-t*F2*F2/(4*M*M))+2*xb*xb*(F1+F2)*(F1+F2))
      +(2-y)*(2-y)*((2+eps*eps)*(4*xb*xb*M*M/t*(1+t/Q2)*(1+t/Q2)+4*(1-xb)*(1+xb*t/Q2))*(F1*F1-t*F2*F2/(4*M*M))
		    +4*xb*xb*(xb+(1-xb+eps*eps/2)*(1-t/Q2)*(1-t/Q2)-xb*(1-2*xb)*t*t/(Q2*Q2))*(F1+F2)*(F1+F2))
      +8*(1+eps*eps)*(1-y-eps*eps*y*y/4)*(2*eps*eps*(1-t/(4*M*M))*(F1*F1-t*F2*F2/(4*M*M))
					  -xb*xb*(1-t/Q2)*(1-t/Q2)*(F1+F2)*(F1+F2));

    cunp_bh[1]=8*sqrt(K2)*(2-y)*((4*xb*xb*M*M/t-2*xb-eps*eps)*(F1*F1-t*F2*F2/(4*M*M))
				 +2*xb*xb*(1-(1-2*xb)*t/Q2)*(F1+F2)*(F1+F2));

    cunp_bh[2]=8*xb*xb*K2*(4*M*M/t*(F1*F1-t*F2*F2/(4*M*M))+2*(F1+F2)*(F1+F2));

    BHUnpC_init=kTRUE;

  }
}

//_____________________________________________________________________________

void TBMK::DVCSUnpC()
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!DVCSUnpC_init) {
    
    alpha_DVCS=2*(2-2*y+y*y);

    C_dvcs_unp=1/((2-xb)*(2-xb))*(4*(1-xb)*(H_Re*H_Re+H_Im*H_Im)
				  -xb*xb*(2*H_Re*E_Re+2*H_Im*E_Im)
				  -(xb*xb+(2-xb)*(2-xb)*t/(4*M*M))*(E_Re*E_Re+E_Im*E_Im));

    DVCSUnpC_init=kTRUE;

  } 
}

//_____________________________________________________________________________

void TBMK::IUnpC(Int_t he)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!IUnpC_init) {

    Double_t F1=F1p(t);  // for now proton target
    Double_t F2=F2p(t);

    alpha_I[0]=-8*(2-y)*((2-y)*(2-y)*K2/(1-y)+t*(1-y)*(2-xb)/Q2);
    alpha_I[1]=-8*(2-y)*t*(1-y)*(2-xb)/Q2;
    alpha_I[2]=-8*sqrt(K2)*(2-2*y+y*y);
    alpha_I[3]=8*sqrt(K2)*he*y*(2-y);

    C_I_unp_Re=F1*H_Re-t*F2*E_Re/(4*M*M);
    C_I_unp_Im=F1*H_Im-t*F2*E_Im/(4*M*M);
    dC_I_unp_Re=-xb*(F1+F2)*(xb*(H_Re+E_Re)/(2-xb))/(2-xb);

    IUnpC_init=kTRUE;

  } 
}

//_____________________________________________________________________________

void TBMK::BH_LC(Int_t he,Int_t hp)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!BH_LC_init) {

    Double_t F1=F1p(t);  // for now proton target
    Double_t F2=F2p(t);

    c_L_bh[0]=8*he*hp*xb*(2-y)*y*sqrt(1+eps*eps)*(F1+F2)/(1-t/(4*M*M))*
              (0.5*(xb/2*(1-t/Q2)-t/(4*M*M))*(2-xb-2*(1-xb)*(1-xb)*t/Q2
					      +eps*eps*(1-t/Q2)-xb*(1-2*xb)*t*t/(Q2*Q2))*(F1+F2)
	       +(1-(1-xb)*t/Q2)*(xb*xb*M*M/t*(1+t/Q2)*(1+t/Q2)+(1-xb)*(1+xb*t/Q2))*(F1+t*F2/(4*M*M)));

    c_L_bh[1]=-8*he*hp*xb*y*sqrt(K2)*sqrt(1+eps*eps)*(F1+F2)/(1-t/(4*M*M))*
              ((t/(2*M*M)-xb*(1-t/Q2))*(1-xb+xb*t/Q2)*(F1+F2)
	       +(1+xb-(3-2*xb)*(1+xb*t/Q2)-4*xb*xb*M*M/t*(1+t*t/(Q2*Q2)))*(F1+t*F2/(4*M*M)));

    BH_LC_init=kTRUE;

//  cout<<"!c_L_bh[0]: "<< c_L_bh[0] << "  c_L_bh[1]: "<< c_L_bh[1] << "    "<< toto  <<endl;

  } 
}

//_____________________________________________________________________________

void TBMK::DVCS_LC(Int_t he,Int_t hp)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!DVCS_LC_init) {

    alpha_DVCS_L=2*he*hp*y*(2-y);

    C_dvcs_L=1/((2-xb)*(2-xb))*(8*(1-xb)*(H_Re*Ht_Re-H_Im*Ht_Im)
				-2*xb*xb*(H_Re*Et_Re-H_Im*Et_Im+Ht_Re*E_Re-Ht_Im*E_Im)
				-xb*(xb*xb/2+(2-xb)*(2-xb)*t/(4*M*M))*(E_Re*Et_Re-E_Im*Et_Im));

    DVCS_LC_init=kTRUE;

  } 
}

//_____________________________________________________________________________

void TBMK::I_LC(Int_t he,Int_t hp)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!I_LC_init) {

    Double_t F1=F1p(t);  // for now proton target
    Double_t F2=F2p(t);

    alpha_I_L[0]=-8*he*hp*y*(((2-y)*(2-y)/(1-y)+2)*K2+t/(1-y)*(2-xb)/Q2);
    alpha_I_L[1]=-8*he*hp*y*t/(1-y)*(2-xb)/Q2;
    alpha_I_L[2]=-8*he*hp*sqrt(K2)*y*(2-y);
    alpha_I_L[3]=8*hp*sqrt(K2)*(2-2*y+y*y);

    C_I_L_Re=xb*(F1+F2)*(H_Re+xb*E_Re/2)/(2-xb)+F1*Ht_Re-xb*(xb*F1/2+t*F2/(4*M*M))*Et_Re/(2-xb);
    C_I_L_Im=xb*(F1+F2)*(H_Im+xb*E_Im/2)/(2-xb)+F1*Ht_Im-xb*(xb*F1/2+t*F2/(4*M*M))*Et_Im/(2-xb);
    dC_I_L_Re=-xb*(F1+F2)*(H_Re+xb*E_Re/2+xb*(Ht_Re+xb*Et_Re/2)/(2-xb))/(2-xb);

    I_LC_init=kTRUE;

  }
}

//_____________________________________________________________________________

void TBMK::BH_TC(Int_t he)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!BH_TC_init) {

    Double_t F1=F1p(t);  // for now proton target
    Double_t F2=F2p(t);

    c_T_bh[0]=-8*he*cos(varphi)*(2-y)*y*sqrt(Q2)/M*sqrt(1+eps*eps)*sqrt(K2)/sqrt(1-y-eps*eps*y*y/4)*(F1+F2)
              *(xb*xb*xb*M*M/Q2*(1-t/Q2)*(F1+F2)
		+(1-(1-xb)*t/Q2)*(xb*xb*M*M/t*(1-t/Q2)*F1+xb*F2/2));

    c_T_bh[1]=-16*he*cos(varphi)*xb*y*sqrt(1-y-eps*eps*y*y/4)*M/sqrt(Q2)*sqrt(1+eps*eps)*(F1+F2)
              *(2*K2*Q2/(t*(1-y-eps*eps*y*y/4))*(xb*(1-t/Q2)*F1+t*F2/(4*M*M))
		+(1+eps*eps)*xb*(1-t/Q2)*(F1+t*F2/(4*M*M)));

    c_T_bh[2]=16*he*sin(varphi)*y*xb*xb*sqrt(1-y-eps*eps*y*y/4)*M/sqrt(Q2)*pow(1+eps*eps,1.5)*(1-t/Q2)*(F1+F2)*(F1+t*F2/(4*M*M));

    BH_TC_init=kTRUE;

  } 
}

//_____________________________________________________________________________

void TBMK::DVCS_TC(Int_t he)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!DVCS_TC_init) {

    alpha_DVCS_T[0]=sqrt(Q2*K2)/(M*sqrt(1-y))*he*y*(2-y)*cos(varphi);

    alpha_DVCS_T[1]=-sqrt(Q2*K2)/(M*sqrt(1-y))*(2-2*y+y*y)*sin(varphi);

    C_dvcs_T[0]=1/((2-xb)*(2-xb))*(4*xb*(H_Re*Et_Re-H_Im*Et_Im)-4*(2-xb)*(Ht_Re*E_Re-Ht_Im*E_Im)+xb*xb*(Et_Re*E_Re-Et_Im*E_Im));

    C_dvcs_T[1]=4/((2-xb)*(2-xb))*((2-xb)*(H_Re*E_Re-H_Im*E_Im)-xb*(Ht_Re*Et_Re-Ht_Im*Et_Im));

    DVCS_TC_init=kTRUE;

  } 
}

//_____________________________________________________________________________

void TBMK::I_TC(Int_t he)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!I_TC_init) {

    Double_t F1=F1p(t);  // for now proton target
    Double_t F2=F2p(t);

    alpha_I_T[0]=-8*M*sqrt(1-y)*sqrt(K2/Q2)*he*y*cos(varphi)*((2-y)*(2-y)/(1-y)+2);
    alpha_I_T[1]=-8*M*sqrt(1-y)*sqrt(K2/Q2)*he*y*cos(varphi);
    alpha_I_T[2]=8*M*sqrt(1-y)*sqrt(K2/Q2)*(2-y)*sin(varphi)*(2-y)*(2-y)/(1-y);
    alpha_I_T[3]=8*M*sqrt(1-y)*sqrt(K2/Q2)*(2-y)*sin(varphi);

    alpha_I_T[4]=-8*M*sqrt((1-y)/Q2)*cos(varphi)*he*y*(2-y);
    alpha_I_T[5]=8*M*sqrt((1-y)/Q2)*cos(varphi)*(2-2*y+y*y);
    alpha_I_T[6]=8*M*sqrt((1-y)/Q2)*sin(varphi)*(2-2*y+y*y);
    alpha_I_T[7]=8*M*sqrt((1-y)/Q2)*sin(varphi)*he*y*(2-y);

    C_I_Tp_Re=(F1+F2)*(xb*xb/(2-xb)*(H_Re+xb*E_Re/2.)+xb*t*E_Re/(4*M*M))
              -xb*xb*F1/(2-xb)*(Ht_Re+xb*Et_Re/2.)
              +t/(4*M*M)*(4*(1-xb)*F2*Ht_Re/(2-xb)-(xb*F1+xb*xb*F2/(2-xb))*Et_Re);

    C_I_Tp_Im=(F1+F2)*(xb*xb/(2-xb)*(H_Im+xb*E_Im/2.)+xb*t*E_Im/(4*M*M))
              -xb*xb*F1/(2-xb)*(Ht_Im+xb*Et_Im/2.)
              +t/(4*M*M)*(4*(1-xb)*F2*Ht_Im/(2-xb)-(xb*F1+xb*xb*F2/(2-xb))*Et_Im);

    C_I_Tm_Re=1/(2-xb)*(xb*xb*F1-(1-xb)*t*F2/(M*M))*H_Re
              +(t/(4*M*M)*((2-xb)*F1+xb*xb*F2/(2-xb))+xb*xb*F1/(2-xb))*E_Re
              -xb*xb*(F1+F2)/(2-xb)*(Ht_Re+t*Et_Re/(4*M*M));

    C_I_Tm_Im=1/(2-xb)*(xb*xb*F1-(1-xb)*t*F2/(M*M))*H_Im
              +(t/(4*M*M)*((2-xb)*F1+xb*xb*F2/(2-xb))+xb*xb*F1/(2-xb))*E_Im
              -xb*xb*(F1+F2)/(2-xb)*(Ht_Im+t*Et_Im/(4*M*M));

    dC_I_Tp_Re=-t/(M*M)*(F2*Ht_Re-xb/(2-xb)*(F1+xb*F2/2.)*Et_Re);

    dC_I_Tm_Im=t/(M*M)*(F2*H_Im-F1*E_Im);

    I_TC_init=kTRUE;

  }
}

//_____________________________________________________________________________

Double_t TBMK::GetBH2(Double_t phi_exp)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

    BHUnpC(); // Calculate coefficients

    TLorentzVector p2b;
    TLorentzVector q2b;
    
    p2b=p2;
    q2b=q2;

    phi_bel=TMath::Pi()-phi_exp*TMath::Pi()/180.;

    q2b.RotateY(-thph);
    p2b.RotateY(-thph);

    q2b.RotateZ(TMath::Pi()-phi_bel);
    p2b.RotateZ(TMath::Pi()-phi_bel);

    q2b.RotateY(thph);
    p2b.RotateY(thph);

    prop1=(k-q2b)*(k-q2b)/Q2;
    prop2=(k+p1-p2b)*(k+p1-p2b)/Q2;

    TBH2=(cunp_bh[0]+cunp_bh[1]*cos(phi_bel)+cunp_bh[2]*cos(2*phi_bel))/(xb*xb*y*y*(1+eps*eps)*(1+eps*eps)*t*prop1*prop2);
   
  return TBH2;
}

//_____________________________________________________________________________

Double_t TBMK::GetDVCS2(Double_t phi_exp)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

    DVCSUnpC(); // Calculate coefficients

    //    phi_bel=TMath::Pi()-phi_exp*TMath::Pi()/180.;

    TDVCS2=(alpha_DVCS*C_dvcs_unp)/(y*y*Q2);

  return TDVCS2;
}

//_____________________________________________________________________________

Double_t TBMK::GetI2(Double_t phi_exp,Int_t he)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!(he==-1||he==0||he==1)) {
    cout << " Error with the electron's polarisation !" << endl;
    exit(-1);
  }

    IUnpC(he); // Calculate coefficients

    TLorentzVector p2b;
    TLorentzVector q2b;
    
    p2b=p2;
    q2b=q2;

    phi_bel=TMath::Pi()-phi_exp*TMath::Pi()/180.;

    q2b.RotateY(-thph);
    p2b.RotateY(-thph);

    q2b.RotateZ(TMath::Pi()-phi_bel);
    p2b.RotateZ(TMath::Pi()-phi_bel);

    q2b.RotateY(thph);
    p2b.RotateY(thph);

    prop1=(k-q2b)*(k-q2b)/Q2;
    prop2=(k+p1-p2b)*(k+p1-p2b)/Q2;

    TI2=(alpha_I[0]*C_I_unp_Re+alpha_I[1]*dC_I_unp_Re+alpha_I[2]*C_I_unp_Re*cos(phi_bel)+alpha_I[3]*C_I_unp_Im*sin(phi_bel))/(xb*y*y*y*t*prop1*prop2);

  return TI2;
}

//_____________________________________________________________________________

Double_t TBMK::GetBH2_L(Double_t phi_exp,Int_t he,Int_t hp)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!(he==-1||he==0||he==1)) {
    cout << " Error with the electron's polarisation !" << endl;
    exit(-1);
  }

  if(!(hp==-1||hp==0||hp==1)) {
    cout << " Error with the proton's polarisation !" << endl;
    exit(-1);
  }

    BH_LC(he,hp); // Calculate coefficients

    TLorentzVector p2b;
    TLorentzVector q2b;
    
    p2b=p2;
    q2b=q2;

    phi_bel=TMath::Pi()-phi_exp*TMath::Pi()/180.;

    q2b.RotateY(-thph);
    p2b.RotateY(-thph);

    q2b.RotateZ(TMath::Pi()-phi_bel);
    p2b.RotateZ(TMath::Pi()-phi_bel);

    q2b.RotateY(thph);
    p2b.RotateY(thph);

    prop1=(k-q2b)*(k-q2b)/Q2;
    prop2=(k+p1-p2b)*(k+p1-p2b)/Q2;

    TBH2_L=(c_L_bh[0]+c_L_bh[1]*cos(phi_bel))/(xb*xb*y*y*(1+eps*eps)*(1+eps*eps)*t*prop1*prop2);
   
  return TBH2_L;
}

//_____________________________________________________________________________

Double_t TBMK::GetDVCS2_L(Double_t phi_exp,Int_t he,Int_t hp)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!(he==-1||he==0||he==1)) {
    cout << " Error with the electron's polarisation !" << endl;
    exit(-1);
  }

  if(!(hp==-1||hp==0||hp==1)) {
    cout << " Error with the proton's polarisation !" << endl;
    exit(-1);
  }

    DVCS_LC(he,hp); // Calculate coefficients

    //    phi_bel=TMath::Pi()-phi_exp*TMath::Pi()/180.;

    TDVCS2_L=(alpha_DVCS_L*C_dvcs_L)/(y*y*Q2);

  return TDVCS2_L;
}

//_____________________________________________________________________________

Double_t TBMK::GetI2_L(Double_t phi_exp,Int_t he,Int_t hp)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!(he==-1||he==0||he==1)) {
    cout << " Error with the electron's polarisation !" << endl;
    exit(-1);
  }

  if(!(hp==-1||hp==0||hp==1)) {
    cout << " Error with the proton's polarisation !" << endl;
    exit(-1);
  }

    I_LC(he,hp); // Calculate coefficients

    TLorentzVector p2b;
    TLorentzVector q2b;
    
    p2b=p2;
    q2b=q2;

    phi_bel=TMath::Pi()-phi_exp*TMath::Pi()/180.;

    q2b.RotateY(-thph);
    p2b.RotateY(-thph);

    q2b.RotateZ(TMath::Pi()-phi_bel);
    p2b.RotateZ(TMath::Pi()-phi_bel);

    q2b.RotateY(thph);
    p2b.RotateY(thph);

    prop1=(k-q2b)*(k-q2b)/Q2;
    prop2=(k+p1-p2b)*(k+p1-p2b)/Q2;

    TI2_L=(alpha_I_L[0]*C_I_L_Re+alpha_I_L[1]*dC_I_L_Re+alpha_I_L[2]*C_I_L_Re*cos(phi_bel)+alpha_I_L[3]*C_I_L_Im*sin(phi_bel))/(xb*y*y*y*t*prop1*prop2);

  return TI2_L;
}

//_____________________________________________________________________________

Double_t TBMK::GetBH2_T(Double_t phi_exp,Int_t he)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!(he==-1||he==0||he==1)) {
    cout << " Error with the electron's polarisation !" << endl;
    exit(-1);
  }

    BH_TC(he); // Calculate coefficients

    TLorentzVector p2b;
    TLorentzVector q2b;
    
    p2b=p2;
    q2b=q2;

    phi_bel=TMath::Pi()-phi_exp*TMath::Pi()/180.;

    q2b.RotateY(-thph);
    p2b.RotateY(-thph);

    q2b.RotateZ(TMath::Pi()-phi_bel);
    p2b.RotateZ(TMath::Pi()-phi_bel);

    q2b.RotateY(thph);
    p2b.RotateY(thph);

    prop1=(k-q2b)*(k-q2b)/Q2;
    prop2=(k+p1-p2b)*(k+p1-p2b)/Q2;

    TBH2_T=(c_T_bh[0]+c_T_bh[1]*cos(phi_bel)+c_T_bh[2]*cos(2*phi_bel))/(xb*xb*y*y*(1+eps*eps)*(1+eps*eps)*t*prop1*prop2);
   
  return TBH2_T;
}

//_____________________________________________________________________________

Double_t TBMK::GetDVCS2_T(Double_t phi_exp,Int_t he)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!(he==-1||he==0||he==1)) {
    cout << " Error with the electron's polarisation !" << endl;
    exit(-1);
  }

    DVCS_TC(he); // Calculate coefficients

    //    phi_bel=TMath::Pi()-phi_exp*TMath::Pi()/180.;

    TDVCS2_T=(alpha_DVCS_T[0]*C_dvcs_T[0]+alpha_DVCS_T[1]*C_dvcs_T[1])/(y*y*Q2);

  return TDVCS2_T;
}

//_____________________________________________________________________________

Double_t TBMK::GetI2_T(Double_t phi_exp,Int_t he)
{
  if(!kin_init) {
    cout << " Kinematic initialization not called yet !" << endl;
    exit(-1);
  }

  if(!(he==-1||he==0||he==1)) {
    cout << " Error with the electron's polarisation !" << endl;
    exit(-1);
  }

    I_TC(he); // Calculate coefficients

    TLorentzVector p2b;
    TLorentzVector q2b;
    
    p2b=p2;
    q2b=q2;

    phi_bel=TMath::Pi()-phi_exp*TMath::Pi()/180.;

    q2b.RotateY(-thph);
    p2b.RotateY(-thph);

    q2b.RotateZ(TMath::Pi()-phi_bel);
    p2b.RotateZ(TMath::Pi()-phi_bel);

    q2b.RotateY(thph);
    p2b.RotateY(thph);

    prop1=(k-q2b)*(k-q2b)/Q2;
    prop2=(k+p1-p2b)*(k+p1-p2b)/Q2;

    TI2_T=(alpha_I_T[0]*C_I_Tp_Re+alpha_I_T[1]*dC_I_Tp_Re+alpha_I_T[2]*C_I_Tm_Im+alpha_I_T[3]*dC_I_Tm_Im
	   +alpha_I_L[4]*C_I_Tp_Re*cos(phi_bel)+alpha_I_L[5]*C_I_Tm_Im*cos(phi_bel)
	   +alpha_I_L[6]*C_I_Tp_Im*sin(phi_bel)+alpha_I_L[7]*C_I_Tm_Re*sin(phi_bel))/(xb*y*y*y*t*prop1*prop2);

  return TI2_T;
}

//_____________________________________________________________________________

void TBMK::GetSigma(Double_t phi_exp,Int_t he,Double_t *sig)
{

  if(!(t1<t&&t<t0)) {
    sig[3]=0;
  }

  if(!(xbmin<xb&&xb<xbmax)) {
    sig[3]=0;
  }

  if((xbmin<xb&&xb<xbmax)&&(t1<t&&t<t0)){
    sig[0]=GetBH2(phi_exp);
    
    sig[1]=GetI2(phi_exp,he);
    
    sig[2]=GetDVCS2(phi_exp);
    
    sig[3]=sig[0]+sig[1]+sig[2];
    
//     for(Int_t i=0;i<4;i++){
      
//       sig[i]*=TMath::Power(alpha,3.)*xb*y/16/TMath::Pi()/TMath::Pi()/Q2/TMath::Sqrt(1.+eps*eps);
      
//       sig[i]*=1./(2.*M*xb*Eb); // dy/dQ2
      
//       sig[i]*=2.*TMath::Pi()*0.38937966e6;
      
//     }
       sig[3]*=TMath::Power(alpha,3.)*xb*y/16/TMath::Pi()/TMath::Pi()/Q2/TMath::Sqrt(1.+eps*eps);      
       sig[3]*=1./(2.*M*xb*Eb); // dy/dQ2     
       sig[3]*=2.*TMath::Pi()*0.38937966e6;
  }

}

//_____________________________________________________________________________

void TBMK::GetSigmaL(Double_t phi_exp, Int_t he, Int_t hp, Double_t *sig)
{

  sig[0]=GetBH2(phi_exp)+GetBH2_L(phi_exp,he,hp);

  sig[1]=GetI2(phi_exp,he)+GetI2_L(phi_exp,he,hp);

  sig[2]=GetDVCS2(phi_exp)+GetDVCS2_L(phi_exp,he,hp);

  sig[3]=sig[0]+sig[1]+sig[2];

//  cout<<" c_L_bh[0]: "<< c_L_bh[0] << "  c_L_bh[1]: "<< c_L_bh[1] <<endl;

  for(Int_t i=0;i<4;i++){

    sig[i]*=TMath::Power(alpha,3.)*xb*y/16/TMath::Pi()/TMath::Pi()/Q2/TMath::Sqrt(1.+eps*eps);

    sig[i]*=1./(2.*M*xb*Eb); // dy/dQ2
    
    sig[i]*=2.*TMath::Pi()*0.38937966e6;

  }
}

//_____________________________________________________________________________

void TBMK::GetSigmaT(Double_t phi_exp, Int_t he, Double_t *sig)
{

  sig[0]=GetBH2(phi_exp)+GetBH2_T(phi_exp,he);

  sig[1]=GetI2(phi_exp,he)+GetI2_T(phi_exp,he);

  sig[2]=GetDVCS2(phi_exp)+GetDVCS2_T(phi_exp,he);

  sig[3]=sig[0]+sig[1]+sig[2];

  for(Int_t i=0;i<4;i++){

    sig[i]*=TMath::Power(alpha,3.)*xb*y/16/TMath::Pi()/TMath::Pi()/Q2/TMath::Sqrt(1.+eps*eps);

    sig[i]*=1./(2.*M*xb*Eb); // dy/dQ2
    
    sig[i]*=0.38937966e6;

  }
}
