//
// TGuzey.cxx, v1.0, Mon Dec 10 11:26:21 CET 2007
// Author: P.Konczykowski
//

#include <iostream>
#include "TGuzey.h"
#include "TLorentzVector.h"

#include <TROOT.h>
#include <TObject.h>
#include <TFile.h>
#include <fstream>

using namespace std;

ClassImp(TGuzey)

////////////////////////////////////////////////////////////////////////////////
//
// Guzey model Class
// 
////////////////////////////////////////////////////////////////////////////////

//_____________________________________________________________________________

 TGuzey::TGuzey()
{
  // Default Constructor

//  cout << " TGuzey default constructor called" << endl << endl;
  Readfile();

}

//_____________________________________________________________________________

 TGuzey::~TGuzey()
{

//  cout << " TGuzey destructor called" << endl << endl;

}


//_____________________________________________________________________________

 void TGuzey::Readfile()
{

  Int_t it,ix,iq;
  Nt=13;
  Nx=27;
  Nq=19;
  NqE=10;

  ifstream in;
  
  // Reading H

  in.open("Hscript_dual_guidal.dat");

  for(it=0;it<Nt;it++){

    for(ix=0;ix<Nx;ix++){
      
      for(iq=0;iq<Nq;iq++){
	
	in >> xB_array[ix] >> Q2_array[iq] >> t_array[it] >> Hscript_Im_array[it][ix][iq] >> Hscript_Re_array[it][ix][iq];

      }
    }
  }

  in.close();

  // Reading E with Ju=0

  in.open("Escript_dual_guidal_Ju00.dat");

  for(it=0;it<Nt;it++){

    for(ix=0;ix<Nx;ix++){
      
      for(iq=0;iq<NqE;iq++){
	
	in >> xB_array[ix] >> Q2E_array[iq] >> t_array[it] >> Escript_Im_array[it][ix][iq] >> Escript_Re_array[it][ix][iq];

      }
    }
  }

  in.close();

}

//_____________________________________________________________________________

 void TGuzey::Interpol()
{
  Int_t it,ix,iq;

  for(it=0;it<Nt;it++){
    for(ix=0;ix<Nx;ix++){ 

      for(iq=0;iq<Nq;iq++){
	Q2tmp_Im[iq]=Hscript_Im_array[it][ix][iq];
	Q2tmp_Re[iq]=Hscript_Re_array[it][ix][iq]; 	
      }
      
      for(iq=0;iq<NqE;iq++){
	Q2Etmp_Im[iq]=Escript_Im_array[it][ix][iq];
	Q2Etmp_Re[iq]=Escript_Re_array[it][ix][iq]; 	
      }
      
      xBtmp_Im[ix]=linear(Nq,Q2_array,Q2tmp_Im,Q2); 
      xBtmp_Re[ix]=linear(Nq,Q2_array,Q2tmp_Re,Q2);   
      
      xBEtmp_Im[ix]=linear(NqE,Q2E_array,Q2Etmp_Im,Q2); 
      xBEtmp_Re[ix]=linear(NqE,Q2E_array,Q2Etmp_Re,Q2);   
      
    }
    
    ttmp_Im[it]=linear(Nx,xB_array,xBtmp_Im,xb);  
    ttmp_Re[it]=linear(Nx,xB_array,xBtmp_Re,xb); 
    
    tEtmp_Im[it]=linear(Nx,xB_array,xBEtmp_Im,xb);  
    tEtmp_Re[it]=linear(Nx,xB_array,xBEtmp_Re,xb); 
    
  }
  
  Hscript_Im=linear(Nt,t_array,ttmp_Im,t); 
  Hscript_Re=linear(Nt,t_array,ttmp_Re,t);  
  
  Escript_Im=linear(Nt,t_array,tEtmp_Im,t); 
  Escript_Re=linear(Nt,t_array,tEtmp_Re,t); 
  
}

//_____________________________________________________________________________

 void TGuzey::GetGPD(Double_t xbh,Double_t Q2h,Double_t th,Double_t *G)
{
  xb=xbh;
  Q2=Q2h;
  t=th;
  
  if(xb<0.01){
     cout <<" !!!!xb is not in the model's domain!!!!"<<endl;
     xb=0.01;
     cout <<"     xb has been replace by 0.01"<<endl; 
  }

  if(xb>0.5){
     //cout <<" !!!!xb is not in the model's domain!!!!"<<endl;
     xb=0.5;
     //cout <<"     xb has been replace by 0.5"<<endl; 
  }

  if(Q2<1.){
     cout <<" !!!!Q2 is not in the model's domain!!!!"<<endl;
     Q2=1.;
     cout <<"     Q2 has been replace by 1"<<endl; 
  }

  if(Q2>10.){
     cout <<" !!!!Q2 is not in the model's domain!!!!"<<endl;
     Q2=10.;
     cout <<"     Q2 has been replace by 10"<<endl; 
  }

  if(t<-1.){
    // cout <<" !!!!t is not in the model's domain!!!!"<<endl;
     t=-1.;
    // cout <<"     t has been replace by -1"<<endl; 
  }

  if(t>-0.005){
     cout <<" !!!!t is not in the model's domain!!!!"<<endl;
     t=-0.005;
     cout <<"     t has been replace by -0.005"<<endl; 
  }

  Interpol();
  
  G[0]=Hscript_Im;
  G[1]=Hscript_Re;
  G[2]=Escript_Im;
  G[3]=Escript_Re;

}
    
//_____________________________________________________________________________

Double_t TGuzey::linear(Int_t n,Double_t *xa,Double_t *ya,Double_t x)
{
  Double_t dift,dif,x1,y1,x2,y2,y;
  Int_t i,ns;
  
  //just in case there is some rounding error

  y=0.;
      
  //Later added by Tobias      
  ns=0;
  
  dif=abs(x-xa[0]);
  
  for(i=0;i<n-1;i++){
    
    // Finding the closest index i
    
    dift=abs(x-xa[i]); 
    if(dift<dif){
      ns=i;
      dif=dift;
    }
  }
  
  // x could be greater or smaller than xa(i)
  
  if (x>xa[ns]){  
    
    y1=ya[ns];
    y2=ya[ns+1];
    x1=xa[ns];
    x2=xa[ns+1]; 
    
    y=((y1-y2)*x+(y2*x1-y1*x2))/(x1-x2);
  }
  else{
    
    y1=ya[ns-1];
    y2=ya[ns];
    x1=xa[ns-1];
    x2=xa[ns]; 

    y=((y1-y2)*x+(y2*x1-y1*x2))/(x1-x2);
  }   
  
  return y;
  
}

//_____________________________________________________________________________
