//
// TGuzey.h, v1.0, Mon Dec 10 11:26:21 CET 2007
// Author: P.Konczykowski
//

#ifndef __TGuzey__
#define __TGuzey__

#include "TObject.h"
#include "TLorentzVector.h"

////////////////////////////////////////////////////////////////////////////////
//
// TGuzey.h
//
// Guzey class
// 
////////////////////////////////////////////////////////////////////////////////

class TGuzey : public TObject
{
  protected :

    Int_t Nt;
    Int_t Nx;
    Int_t Nq;
    Int_t NqE;

    Double_t xb;
    Double_t Q2;
    Double_t t;

    // Arrays of variables from the read data files
    Double_t xB_array[27],Q2_array[19],t_array[13];
    Double_t Hscript_Im_array[13][27][19],Hscript_Re_array[13][27][19];

    // Temporary arrays used for 3D interpolation
    Double_t Q2tmp_Im[19],Q2tmp_Re[19];
    Double_t xBtmp_Im[27],xBtmp_Re[27];
    Double_t ttmp_Im[13],ttmp_Re[13];

    // The same for the GPD E
    Double_t Escript_Im_array[13][27][10],Escript_Re_array[13][27][10];
    Double_t Q2Etmp_Im[10],Q2Etmp_Re[10],Q2E_array[10];
    Double_t xBEtmp_Im[27],xBEtmp_Re[27];
    Double_t tEtmp_Im[13],tEtmp_Re[13];

    //Interpolation results
    Double_t Hscript_Im,Hscript_Re,Escript_Im,Escript_Re;

  public :

    TGuzey();
    virtual ~TGuzey();

    void Readfile();

    void Interpol();

    void GetGPD(Double_t xbh,Double_t Q2h,Double_t th,Double_t *G);
    /*   G[0]=Hscript_Im; */
    /*   G[1]=Hscript_Re; */
    /*   G[2]=Escript_Im; */
    /*   G[3]=Escript_Re; */

    Double_t linear(Int_t n,Double_t *xa,Double_t *ya,Double_t x);

    ClassDef(TGuzey,1) //Guzey Base Class
}; // End of TGuzey class definition

#endif
