#include "TROOT.h"
#include "TObject.h"
#include "TSystem.h"
#include "TUnixSystem.h"
#include "TFile.h"
#include "TMath.h"
#include "TLorentzVector.h"
#include "TString.h"
#include "TH1.h"
#include "TH2.h"
#include <iostream>
#include <fstream>
#include <TRandom.h>

#include "TBMK.h"
#include "TGuzey.h"

using namespace std;

TROOT root("whatt","This is just for the ROOT object");

int main(int argc,char* argv[])
{
   ofstream out;

   out.open("eventclas_Guzey.out");

   Double_t r,pond;
   Double_t mp,E,Ep,thl,thph,ql,q0,qp0,p0,pp0;
   Double_t p1cm,p3cm,thppcm,t0,t1,t,psf;
   Double_t Q2,xb,xbmin,xbmax,s,nu,pi,phi,phil;
   Int_t i,N;

   Double_t sigma[4];

   TBMK piotr(5.75);

   E=5.75;
   mp=0.938272;
   pi=3.14159265;

   N=0;

   TRandom *ran=new TRandom();

   TLorentzVector klab,kplab,qlab,qplab,plab,pplab;
   TLorentzVector kvir,kpvir,qvir,qpvir,pvir,ppvir;
   TLorentzVector kcdm,kpcdm,qcdm,qpcdm,pcdm,ppcdm;
   TVector3 b;

   for(i=0;i<20000;)
   {
   Q2=ran->Uniform(1.,4.);

   xbmin=2.*E*Q2/(mp*(4.*E*E-Q2))+0.00001;
   xbmax=Q2/(Q2+4.-mp*mp)-0.00001;
   if(xbmin<0.01)xbmin=0.01;
   if(xbmax>0.5)xbmax=0.5;
   xb=ran->Uniform(xbmin,xbmax);
   
   phi=ran->Uniform(0,2.*pi);
   phil=ran->Uniform(0,2.*pi);

   nu=Q2/(2.*mp*xb);   
   s=-Q2+mp*mp+2.*mp*nu;
   Ep=E-Q2/(2.*mp*xb);

   thl=2.*asin(sqrt(Q2/(4.*E*Ep)));
   ql=sqrt(E*E+Ep*Ep-2.*E*Ep*cos(thl));
   thph=asin(-Ep*sin(thl)/ql);
   q0=(s-Q2-mp*mp)/(2.*sqrt(s));
   qp0=(s-mp*mp)/(2.*sqrt(s));

   p0=(s+Q2+mp*mp)/(2.*sqrt(s));
   pp0=(s+mp*mp)/(2.*sqrt(s));

   p1cm=sqrt(Q2+q0*q0);
   p3cm=qp0;

   t0=Q2*Q2/(4.*s)-(p1cm-p3cm)*(p1cm-p3cm)-0.00001;
   t1=Q2*Q2/(4.*s)-(p1cm+p3cm)*(p1cm+p3cm)+0.00001;
   if(t1<-1)t1=-1;
   if(t0>-0.005)t0=-0.005;
   //t=ran->Uniform(-1,-0.005);
   t=ran->Uniform(t1,t0);

   psf=2*TMath::Pi()*3*(xbmax-xbmin)*(t0-t1);

   thppcm=2.*asin(sqrt((t0-t)/(4.*p1cm*p3cm)));

   b.SetXYZ(0.,0.,ql/(nu+mp));

   //Ref du labo
   klab.SetPxPyPzE(0.,0.,E,E);
   kplab.SetPxPyPzE(Ep*sin(thl),0.,Ep*cos(thl),Ep); 
   plab.SetPxPyPzE(0.,0.,0.,mp);

   //Ref du cdm
   qpcdm.SetPxPyPzE(qp0*sin(thppcm),0.,qp0*cos(thppcm),qp0);
   ppcdm.SetPxPyPzE(-qp0*sin(thppcm),0.,-qp0*cos(thppcm),pp0);

   //Ref du photon virtuel
   qpvir=qpcdm;
   qpvir.Boost(b);
   qpvir.RotateZ(phi);
   ppvir=ppcdm;
   ppvir.Boost(b);
   ppvir.RotateZ(phi);

   //on trouve enfin les composantes des 4-vect des pces sortantes dans le ref du lab
   kplab.RotateZ(phil);
   qplab=qpvir;
   qplab.RotateY(thph);
   qplab.RotateZ(phil);
   pplab=ppvir;
   pplab.RotateY(thph);
   pplab.RotateZ(phil);

   piotr.Kin(xb,Q2,t);
	    
   piotr.GetSigma(0,0,sigma);

   //pond=(1/Q2)*(1/Q2)*pow(1.-t/0.71,-4.)*pow(1.+(xb-0.3)/0.3,-1.)*(1.-0.4*(1.-cos(phi)));
   pond=sigma[3];
   r=ran->Uniform(0.,20.);

   if(r<pond) 
     { 
       out<<xb<<" "<<Q2<<" "<<t<<" "<<phi<<" "<<phil<<" "<<kplab(0)<<" "<<kplab(1)<<" "<<kplab(2)<<" "<<qplab(0)<<" "<<qplab(1)<<" "<<qplab(2)<<" "<<pplab(0)<<" "<<pplab(1)<<" "<<pplab(2)<<" "<<psf<<endl;
    i++;
    if(!(i%1000))cout<<" i: "<<i<<" N: "<<N<<endl;

//   if(xb<0.01){
//      cout <<" !!!!xb is not in the model's domain!!!!"<<endl;
//   }

//   if(xb>0.5){
//      cout <<" !!!!xb is not in the model's domain!!!!"<<endl;
//   }

//   if(Q2<1.){
//      cout <<" !!!!Q2 is not in the model's domain!!!!"<<endl;
//   }

//   if(Q2>10.){
//      cout <<" !!!!Q2 is not in the model's domain!!!!"<<endl;
//   }

//   if(t<-1.){
//      cout <<" !!!!t is not in the model's domain!!!!"<<endl;
//   }

//   if(t>-0.005){
//      cout <<" !!!!t is not in the model's domain!!!!"<<endl;
//   }

  //cout<<i<<endl;
  //cout<<xbmin<<"   "<<xbmax<<"    "<<t0<<"   "<<t1<<"   "<<psf<<endl;
     } 

   N++;  
   } 	
   cout<<" N: "<< N <<endl;
   out.close();
}
